;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xca7375d8f89c63096bda59acb2b1c918 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x82cc29ae3ac73c8d2d26ecee53188792 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x794b0f31ff8145fe93600cb88bbe9b66 . "./code.cg2js/cg-toolbar.fasl")
(#x7131346cae7a8399f8410747c0d8e51a . "./code.cg2js/cg-chart-or-plot.fasl")
(#x2d9e543606e0bac8bf7e456fc4414dea . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x8cc88986ba43f4f6da676c4f23dbea3c . "./code.cg2js/cg-class-grid.fasl")
(#x768a8caad1b39013be01827356d7b453 . "./code.cg2js/cg-tray.fasl")
(#x8ebc72e6e90ab7d002692d60df95c946 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x47ed6dbbd52f6953aef64c3ba8cb34d1 . "./code.cg2js/cg-class-support.fasl")
(#x2a87dbc70256221e635e6fb07a4d2938 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x484871083805436cfa94e36493fcf653 . "./code.cg2js/cg-dropping-outline.fasl")
(#x989fdd9a3a017496aae65e5eb4696454 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x547e4ff3d65d3ec406951ade47ec1e8b . "./code.cg2js/cg-choice-list.fasl")
(#xf168227432e2005555889e31165e6fd6 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x61f80471df34deb4592320e16bde5667 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xa29ee9fe4a7fb51b347bae384f3639b0 . "./code.cg2js/cg-timer.fasl")
(#xdf03a4322fba2d3bfbec483c9a3cd713 . "./code.cg2js/cg-directory-dialog.fasl")
(#x1620862ce7b326e5b143cbb38260fa77 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x5a9f5420930bc7ba2483d769d8bfe730 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x461110aa58971934a778f83e08415ca5 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x8da699b24eae80509368421fdbe65475 . "./code.cg2js/cg-html-widget.fasl")
(#x9f21eaf1fe0ba6c7cb6655fa37bbdff4 . "./code.cg2js/cg-selected-object.fasl")
(#x290aed2533de67e575040f622a36380c . "./code.cg2js/cg-lamp.fasl")
(#x9763c82af51f68503020516a28df6eea . "./code.cg2js/cg-scroll-bar.fasl")
(#x393569e01117b9e376cfbb7ccd4ffd8e . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x8b6d689c9cc57c2d8b4331a89ed43fc7 . "./code.cg2js/cg-message-dialog.fasl")
(#x43da17dd45ec2cee21de8a942bbc4777 . "./code.cg2js/cg-dde-utils.fasl")
(#xe0f6258d1d2944b05af60967627f1227 . "./code.cg2js/cg-group-box.fasl")
(#x2691cbedde8d050963e2fa66b38eb896 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xa94abb1d26627be6d5f218c466141d80 . "./code.cg2js/cg-split-bar.fasl")
(#x562f268ca5811f9942cd98291900d575 . "./code.cg2js/cg-radio-button.fasl")
(#x40f9f080b7d80672f279b5e420457313 . "./code.cg2js/cg-base.fasl")
(#x42bbd5636610313db669d118d21c4eb0 . "./code.cg2js/cg-paren-matching.fasl")
(#xa84d405c2bb329df622a5d439567395a . "./code.cg2js/cg-graphics-context.fasl")
(#x44d0106df4dd0ee996d91bade786c813 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x3c5ed4b441ad495d3704e54fe4e419db . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xc62b834a25c8bed4cb30ec01c3acbd2b . "./code.cg2js/cg-plot-widget.fasl")
(#x80a9af498f25f40438353a73f23b62ca . "./code.cg2js/cg-dev.fasl")
(#x5fe0db47e823df211d3efaee57216e0e . "./code.cg2js/cg-text-edit-pane.fasl")
(#x2bdbf1a0447e3f94d608c5fa7b9f8482 . "./code.cg2js/cg-tab-control.fasl")
(#xa8c4dac242cf6faa30b2e205b8c165f0 . "./code.cg2js/cg-fill-texture.fasl")
(#x7344ac92179c2f858e761f235e4c8dbc . "./code.cg2js/cg-menu-tooltip.fasl")
(#xee31d2e1eb0067431588758cf599caa5 . "./code.cg2js/cg-trackbar.fasl")
(#x493d8a158512fe91443e821493dc8071 . "./code.cg2js/cg-nodes.fasl")
(#xd5d07af84e42dfd12e5af0b6926f1438 . "./code.cg2js/cg-common-control.fasl")
(#x1e3d0ecf561dbbef31693ea680ec4dce . "./code.cg2js/cg-list-view.fasl")
(#xa7b9e62cfad867f1ae83ccdaa8c85fa0 . "./code.cg2js/cg-rich-edit.fasl")
(#xd3c34ac7620aa7e0a3348cdc7a21664e . "./code.cg2js/cg-project-window.fasl")
(#x29fd301e7d52b630364d7a50a7814492 . "./code.cg2js/cg-item-list.fasl")
(#xb903f6dede01c58b7ca100fec9581be7 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x6cd7e5741b6ec4c07ece6d42f6e6c74a . "./code.cg2js/cg-get-position.fasl")
(#xedf23704ced8dca15239af5b6da2c6e9 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x1595ac4e4fc2885429966386f8f0f27e . "./code.cg2js/cg-drawable.fasl")
(#x2dc226bba6fcaf3df8d55a43ce0fde32 . "./code.cg2js/cg-lettered-menu.fasl")
(#xb6e1baac779eaf9d6bccde3e64df65d1 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x80080bd360f9dc30b64854ceee6b1795 . "./code.cg2js/cg-html-dialog.fasl")
(#xa0fecc8a2d95461881cc00a7bde354f3 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x69c5140bbb173acb589fdbf5dc7eea8b . "./code.cg2js/cg-utility-dialog.fasl")
(#xc6f556ea8b93ff3906794830497c6be1 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x02ea71cbb896e7408d870646d5e5c634 . "./code.cg2js/cg-property.fasl")
(#xcbf503eeb0e9239b1a4d5e26c641ca5c . "./code.cg2js/cg-menu.fasl")
(#x25499b3752604035713d822476f63b05 . "./code.cg2js/cg-clipboard.fasl")
(#x42a2c91e3f9b20a9e73d5801e9c237f6 . "./code.cg2js/cg-toggling-widget.fasl")
(#x8d0ad2b70868af80bc5513d176983aa9 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x3d46e3493391bc8cbb541cdbf2c4cdaf . "./code.cg2js/cg-template-string.fasl")
(#x84a41b761e9a30f69f40e6d8306c5142 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xa9302ce739ae2c854cbed3ae323db226 . "./code.cg2js/cg-os-window.fasl")
(#x6e536521d6c363eb3cde410d06f924aa . "./code.cg2js/cg-grid-widget.fasl")
(#xa3243a1f4f88823c32de16b791d7dc21 . "./code.cg2js/cg-picture-widget.fasl")
(#x789b89ab94f85ae533d68716ade9a754 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x0c9e927fca8d8536d6ca98e92341c1c5 . "./code.cg2js/cg-static-text.fasl")
(#x9903750400115fe161e826b11516794e . "./code.cg2js/cg-clipboard-stack.fasl")
(#x46ed7234f2029a53fe58c7e2ba8fe01f . "./code.cg2js/cg-text-widget.fasl")
(#xa06d8b47ae93111696be18d2e1c6f256 . "./code.cg2js/cg-calendar.fasl")
(#xe1c1fd069ce40bbbf1dfe09436b0c5c4 . "./code.cg2js/cg-check-box.fasl")
(#xb520eaca5b189133b87a8c6e1eaa6b2b . "./code.cg2js/cg-acache.fasl")
(#xb56708e9e32a7671a22a59910e4971e1 . "./code.cg2js/cg-wrap-string.fasl")
(#xca02bcd354ffae9f73162c25570263fc . "./code.cg2js/cg-bitmap-stream.fasl")
(#x1d55a1204e8599839f1066866da1dd17 . "./code.cg2js/cg-font-dialog.fasl")
(#x588f4b4ab819fb5fb4f292cae35aad5d . "./code.cg2js/cg-outline.fasl")
(#xb33279e138c91b1a9ed662b0201a6bb3 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x840b2c08a2760e27a279a39af8afa09e . "./code.cg2js/cg-status-bar.fasl")
(#x63c84a60f0cd7ad21a6ec22b60f2d9c7 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xdf4534677894593db43293f0abf47e0c . "./code.cg2js/cg-hotspot.fasl")
(#x776bbbfde8da85537e67a1a420487fc7 . "./code.cg2js/cg-scaling-stream.fasl")
(#x59a3432eb8174a1faae2bbdd2fa63f72 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x4cb37bf9392da13d583e73ab64b4ac0c . "./code.cg2js/cg-file-dialog.fasl")
(#xdc11bf5308b302f80aa1b99909749da3 . "./code.cg2js/cg-color-dialog.fasl")
(#x519bda03a4d679b179d525f185f18efe . "./code.cg2js/cg-get-pixmap.fasl")
(#x717958ad38be73c981256c102b87412f . "./code.cg2js/cg-object-editor.fasl")
(#xb6cc00cf6309d43aef63ab2b9f11f3d4 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x504338947347e0733769a9dcf08b32c0 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x761529ecfa6998342c872a85fca95084 . "./code.cg2js/cg-pixmap.fasl")
(#x04d8948d3c336f0e3857c86812facac5 . "./code.cg2js/cg-intersect.fasl")
(#x269e766cd7687e9346abfe9b3c0805b7 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x9f392003d46a9ed16bc9bf6a7139cff7 . "./code.cg2js/cg-icon.fasl")
(#x3b8e716a8bec6376f4706413e46525ae . "./code.cg2js/cg-yes-no-string.fasl")
(#x58a0aca860ecf536a9a369fffca15b29 . "./code.cg2js/cg-text-or-combo.fasl")
(#x38a27d84a664d2f8ceb52c812a384939 . "./code.cg2js/cg.fasl")
(#x4739bffa37389dd9e9cb72947d059a7d . "./code.cg2js/cg-dialog-item.fasl")
(#x63f6176a72dadd1cb037870f4da286b0 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xf602c92341a2932174987efb0aaf0101 . "./code.cg2js/cg-caret.fasl")
(#x9353188afd06bdd413d304259607d89d . "./code.cg2js/cg-editable-text.fasl")
(#x1644bbbace909c43a9d7076337f7b4d1 . "./code.cg2js/cg-choose-printer.fasl")
(#x2ffdd14b37531afd8d11d2896678750c . "./code.cg2js/cg-yes-no-list.fasl")
(#x876734fac363b4feb39620d41f9444c9 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x251873d59e37abddb0e24466ab145e84 . "./code.cg2js/cg-tooltip.fasl")
(#x34abe8cc6f43fe17f1d7e4cfbf9843dd . "./code.cg2js/cg-class-slot-grid.fasl")
(#xfb873c04ad191965a9aa18406ac656b7 . "./code.cg2js/cg-combo-box.fasl")
(#xdb77a12d69f3fccd0d718f179d3b82c7 . "./code.cg2js/cg-curve.fasl")
(#x8b25790d69bea2ee9845597729ca9afd . "./code.cg2js/cg-web-browser.fasl")
(#xf4298aae2da8c2e674728679230c5b32 . "./code.cg2js/cg-printing.fasl")
(#xbc634418000ed30ebf567f63ed8f0b6e . "./code.cg2js/cg-lisp-widget.fasl")
(#x4a03979776f9cf91b5637f91f80fc1b7 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xbbacfe10a019d3eebac0d3b1e238419c . "./code.cg2js/cg-palette.fasl")
(#xd3467e693bc54c2c59f5a8af1f094f8f . "./code.cg2js/cg-os-widget.fasl")
(#xdc08234aa2bb41cd0b03b1d546ec7c15 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x8e9510ecf8ff76a02467e1b0dfc7eeb4 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xac736c79487d4f3c5289a381e82dc7a7 . "./code.cg2js/cg-button.fasl")
(#xd7768911eef0522edfee717544787e9e . "./code.cg2js/cg-edit-in-place.fasl")
(#xb02aadec5df71c240c3b74c6d97f9b90 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xa4edc593e9e9355fd10eb81319f93377 . "./code.cg2js/cg-comtab.fasl")
(#xb2f1c8d981fef518cbc2a448970835c1 . "./code.cg2js/cg-string-dialog.fasl")
(#x183b55f0695ae5e05ebed1d2ab2051e4 . "./code.cg2js/cg-header-control.fasl")
(#xbf9696934adf46724748262f6847fd84 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x0adb0ccc865986ed750368b8fca9d029 . "./code.cg2js/cg-up-down-control.fasl")
(#xc9cc45c427c3cb78166717f73c273bd9 . "./code.cg2js/cg-layout.fasl")
(#xc9cf5c10b336a44db068bdc8127054e3 . "./code.cg2js/cg-chart-widget.fasl")
(#x382a9182f82d30e035a04ccf3209b5d8 . "./code.cg2js/cg-lisp-text.fasl")
(#xf5853776d518a86373931944e2645435 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x6e8c40a5d95deb343333e75810132460 . "./code.cg2js/cg-progress-indicator.fasl")
(#x5630b898f6b7698ff25802ddaf021eae . "./code.idejs/ide-inspector.fasl")
(#xfd984112869974d0e7f40dfcafdeebeb . "./code.idejs/ide-grapher.fasl")
(#x792ae1d194370954028f80edc01e72b2 . "./code.idejs/ide-base.fasl")
(#xe36ca397a9880542ad451134e4debd10 . "./code.idejs/ide-keyshort.fasl")
(#x7c88abf90f7d1be87903401ebde73470 . "./code.idejs/ide-dev.fasl")
(#xd4e6a1a94c34ad7c671a27a64de5f65c . "./code.idejs/ide-class-browser.fasl")
(#x877d50454335945fb135f97fb9fa2152 . "./code.idejs/ide-package-list.fasl")
(#x7823c32fea9dfd9d2e73a81db3e9b7a0 . "./code.idejs/ide-intro.fasl")
(#xd59ad525b1e5d162a26be90694a9be3f . "./code.idejs/ide-class-grapher.fasl")
(#xfb0b0fa55aec2b3a810db46f84ab31d4 . "./code.idejs/ide-window-list.fasl")
(#x033f13595731bb5065fc3eeec4382d69 . "./code.idejs/ide-find-definitions.fasl")
(#x9a1a8ec19b3c609716358b8c41f734c8 . "./code.idejs/ide-trace.fasl")
(#x7f8546b3d3b389f08364b44e186c9a45 . "./code.idejs/ide-editor.fasl")
(#xbef5b3b24febfbe62552e8d540e7ecb3 . "./code.idejs/ide-patch-dialog.fasl")
(#x882a2b4f6b2db922ce4b1f3e0c59c53d . "./code.idejs/ide-project.fasl")
(#xaf06074120e13c2ab77aae0a8f0b5f8c . "./code.idejs/ide.fasl")
(#x638a308bf1f1293e3533f335b6b428d6 . "./code.idejs/ide-apropos.fasl")
(#x4d9ed2c8e5a5d9637ce42bde83c53d55 . "./code.idejs/ide-acache.fasl")
(#x117e8630b74571240da8a7669712cfc2 . "./code.idejs/ide-clipboard.fasl")
(#xf44fc4b658931eeaf133a15af8628a38 . "./code.idejs/ide-process-browser.fasl")
(#x520f6daf6294bfc40c03e57b64b8ddc4 . "./code.idejs/ide-find-in-files.fasl")
(#x40bbdda1b95c708a38459b8456e86e27 . "./code.idejs/ide-form.fasl")
(#x92416cb8814e8a59af9bd6658d239971 . "./code.idejs/ide-navigator.fasl")
(#xb19a66ec109be5a69fb5293c577fc104 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
